searchData={"items":[{"type":"module","title":"NimbleOptions","doc":"Provides a standard API to handle keyword-list-based options.\n\n`NimbleOptions` allows developers to create schemas using a\npre-defined set of options and types. The main benefits are:\n\n  * A single unified way to define simple static options\n  * Config validation against schemas\n  * Automatic doc generation","ref":"NimbleOptions.html"},{"type":"module","title":"Schema Options - NimbleOptions","doc":"These are the options supported in a *schema*. They are what\ndefines the validation for the items in the given schema.\n\n* `:type` - The type of the option item. The default value is `:any`.\n\n* `:required` (`t:boolean/0`) - Defines if the option item is required. The default value is `false`.\n\n* `:default` (`t:term/0`) - The default value for the option item if that option is not specified. This value\n  is *validated* according to the given `:type`. This means that you cannot\n  have, for example, `type: :integer` and use `default: \"a string\"`.\n\n* `:keys` (`t:keyword/0`) - Available for types `:keyword_list`, `:non_empty_keyword_list`, and `:map`,\n  it defines which set of keys are accepted for the option item. The value of the\n  `:keys` option is a schema itself. For example: `keys: [foo: [type: :atom]]`.\n  Use `:*` as the key to allow multiple arbitrary keys and specify their schema:\n  `keys: [*: [type: :integer]]`.\n\n* `:deprecated` (`t:String.t/0`) - Defines a message to indicate that the option item is deprecated. The message will be displayed as a warning when passing the item.\n\n* `:doc` (`t:String.t/0` or `false`) - The documentation for the option item.\n\n* `:subsection` (`t:String.t/0`) - The title of separate subsection of the options' documentation\n\n* `:type_doc` (`t:String.t/0` or `false`) - The type doc to use *in the documentation* for the option item. If `false`,\n  no type documentation is added to the item. If it's a string, it can be\n  anything. For example, you can use `\"a list of PIDs\"`, or you can use\n  a typespec reference that ExDoc can link to the type definition, such as\n  `` \"`t:binary/0`\" ``. You can use Markdown in this documentation. If the\n  `:type_doc` option is not present, NimbleOptions tries to produce a type\n  documentation automatically if it can do it unambiguously. For example,\n  if `type: :integer`, NimbleOptions will use `t:integer/0` as the\n  auto-generated type doc.\n\n* `:type_spec` (`t:Macro.t/0`) - The quoted spec to use *in the typespec* for the option item. You should use this\n  when the auto-generated spec is not specific enough. For example, if you are performing\n  custom validation on an option (with the `{:custom, ...}` type), then the\n  generated type spec for that option will always be `t:term/0`, but you can use\n  this option to customize that. The value for this option **must** be a quoted Elixir\n  term. For example, if you have an `:exception` option that is validated with a\n  `{:custom, ...}` type (based on `is_exception/1`), you can override the type\n  spec for that option to be `quote(do: Exception.t())`. *Available since v1.1.0*.","ref":"NimbleOptions.html#module-schema-options"},{"type":"module","title":"Types - NimbleOptions","doc":"* `:any` - Any type.\n\n  * `:keyword_list` - A keyword list.\n\n  * `:non_empty_keyword_list` - A non-empty keyword list.\n\n  * `:map` - A map consisting of `:atom` keys. Shorthand for `{:map, :atom, :any}`.\n    Keys can be specified using the `keys` option.\n\n  * `{:map, key_type, value_type}` - A map consisting of `key_type` keys and\n    `value_type` values.\n\n  * `:atom` - An atom.\n\n  * `:string` - A string.\n\n  * `:boolean` - A boolean.\n\n  * `:integer` - An integer.\n\n  * `:non_neg_integer` - A non-negative integer.\n\n  * `:pos_integer` - A positive integer.\n\n  * `:float` - A float.\n\n  * `:timeout` - A non-negative integer or the atom `:infinity`.\n\n  * `:pid` - A PID (process identifier).\n\n  * `:reference` - A reference (see `t:reference/0`).\n\n  * `nil` - The value `nil` itself. Available since v1.0.0.\n\n  * `:mfa` - A named function in the format `{module, function, arity}` where\n    `arity` is a list of arguments. For example, `{MyModule, :my_fun, [arg1, arg2]}`.\n\n  * `:mod_arg` - A module along with arguments, such as `{MyModule, arguments}`.\n    Usually used for process initialization using `start_link` and similar. The\n    second element of the tuple can be any term.\n\n  * `{:fun, arity}` - Any function with the specified arity.\n\n  * `{:in, choices}` - A value that is a member of one of the `choices`. `choices`\n    should be a list of terms or a `Range`. The value is an element in said\n    list of terms, that is, `value in choices` is `true`. This was previously\n    called `:one_of` and the `:in` name is available since version 0.3.3 (`:one_of`\n    has been removed in v0.4.0).\n\n  * `{:custom, mod, fun, args}` - A custom type. The related value must be validated\n    by `mod.fun(values, ...args)`. The function should return `{:ok, value}` or\n    `{:error, message}`.\n\n  * `{:or, subtypes}` - A value that matches one of the given `subtypes`. The value is\n    matched against the subtypes in the order specified in the list of `subtypes`. If\n    one of the subtypes matches and **updates** (casts) the given value, the updated\n    value is used. For example: `{:or, [:string, :boolean, {:fun, 2}]}`. If one of the\n    subtypes is a keyword list or map, you won't be able to pass `:keys` directly. For this reason,\n    `:keyword_list`, `:non_empty_keyword_list`, and `:map` are special cased and can\n    be used as subtypes with `{:keyword_list, keys}`, `{:non_empty_keyword_list, keys}` or `{:map, keys}`.\n    For example, a type such as `{:or, [:boolean, keyword_list: [enabled: [type: :boolean]]]}`\n    would match either a boolean or a keyword list with the `:enabled` boolean option in it.\n\n  * `{:list, subtype}` - A list where all elements match `subtype`. `subtype` can be any\n    of the accepted types listed here. Empty lists are allowed. The resulting validated list\n    contains the validated (and possibly updated) elements, each as returned after validation\n    through `subtype`. For example, if `subtype` is a custom validator function that returns\n    an updated value, then that updated value is used in the resulting list. Validation\n    fails at the *first* element that is invalid according to `subtype`. If `subtype` is\n    a keyword list or map, you won't be able to pass `:keys` directly. For this reason,\n    `:keyword_list`, `:non_empty_keyword_list`, and `:map` are special cased and can\n    be used as the subtype by using `{:keyword_list, keys}`, `{:non_empty_keyword_list, keys}`\n    or `{:keyword_list, keys}`. For example, a type such as\n    `{:list, {:keyword_list, enabled: [type: :boolean]}}` would a *list of keyword lists*,\n    where each keyword list in the list could have the `:enabled` boolean option in it.\n\n  * `{:tuple, list_of_subtypes}` - A tuple as described by `tuple_of_subtypes`.\n    `list_of_subtypes` must be a list with the same length as the expected tuple.\n    Each of the list's elements must be a subtype that should match the given element in that\n    same position. For example, to describe 3-element tuples with an atom, a string, and\n    a list of integers you would use the type `{:tuple, [:atom, :string, {:list, :integer}]}`.\n    *Available since v0.4.1*.\n\n  * `{:struct, struct_name}` - An instance of the struct type given.","ref":"NimbleOptions.html#module-types"},{"type":"module","title":"Example - NimbleOptions","doc":"iex> schema = [\n    ...>   producer: [\n    ...>     type: :non_empty_keyword_list,\n    ...>     required: true,\n    ...>     keys: [\n    ...>       module: [required: true, type: :mod_arg],\n    ...>       concurrency: [\n    ...>         type: :pos_integer,\n    ...>       ]\n    ...>     ]\n    ...>   ]\n    ...> ]\n    ...>\n    ...> config = [\n    ...>   producer: [\n    ...>     concurrency: 1,\n    ...>   ]\n    ...> ]\n    ...>\n    ...> {:error, %NimbleOptions.ValidationError{} = error} = NimbleOptions.validate(config, schema)\n    ...> Exception.message(error)\n    \"required :module option not found, received options: [:concurrency] (in options [:producer])\"","ref":"NimbleOptions.html#module-example"},{"type":"module","title":"Nested Option Items - NimbleOptions","doc":"`NimbleOptions` allows option items to be nested so you can recursively validate\nany item down the options tree.","ref":"NimbleOptions.html#module-nested-option-items"},{"type":"module","title":"Example - NimbleOptions","doc":"iex> schema = [\n    ...>   producer: [\n    ...>     required: true,\n    ...>     type: :non_empty_keyword_list,\n    ...>     keys: [\n    ...>       rate_limiting: [\n    ...>         type: :non_empty_keyword_list,\n    ...>         keys: [\n    ...>           interval: [required: true, type: :pos_integer]\n    ...>         ]\n    ...>       ]\n    ...>     ]\n    ...>   ]\n    ...> ]\n    ...>\n    ...> config = [\n    ...>   producer: [\n    ...>     rate_limiting: [\n    ...>       interval: :oops!\n    ...>     ]\n    ...>   ]\n    ...> ]\n    ...>\n    ...> {:error, %NimbleOptions.ValidationError{} = error} = NimbleOptions.validate(config, schema)\n    ...> Exception.message(error)\n    \"invalid value for :interval option: expected positive integer, got: :oops! (in options [:producer, :rate_limiting])\"","ref":"NimbleOptions.html#module-example"},{"type":"module","title":"Validating Schemas - NimbleOptions","doc":"Each time `validate/2` is called, the given schema itself will be validated before validating\nthe options.\n\nIn most applications the schema will never change but validating options will be done\nrepeatedly.\n\nTo avoid the extra cost of validating the schema, it is possible to validate the schema once,\nand then use that valid schema directly. This is done by using the `new!/1` function first, and\nthen passing the returned schema to `validate/2`.\n\n> #### Create the Schema at Compile Time {: .tip}\n>\n> If your option schema doesn't include any runtime-only terms in it (such as anonymous\n> functions), you can call `new!/1` to validate the schema and returned a *compiled* schema\n> **at compile time**. This is an efficient way to avoid doing any unnecessary work at\n> runtime. See the example below for more information.","ref":"NimbleOptions.html#module-validating-schemas"},{"type":"module","title":"Example - NimbleOptions","doc":"iex> raw_schema = [\n    ...>   hostname: [\n    ...>     required: true,\n    ...>     type: :string\n    ...>   ]\n    ...> ]\n    ...>\n    ...> schema = NimbleOptions.new!(raw_schema)\n    ...> NimbleOptions.validate([hostname: \"elixir-lang.org\"], schema)\n    {:ok, hostname: \"elixir-lang.org\"}\n\nCalling `new!/1` from a function that receives options will still validate the schema each time\nthat function is called. Declaring the schema as a module attribute is supported:\n\n    @options_schema NimbleOptions.new!([...])\n\nThis schema will be validated at compile time. Calling `docs/1` on that schema is also\nsupported.","ref":"NimbleOptions.html#module-example"},{"type":"function","title":"NimbleOptions.docs/2","doc":"Returns documentation for the given schema.\n\nYou can use this to inject documentation in your docstrings. For example,\nsay you have your schema in a module attribute:\n\n    @options_schema [...]\n\nWith this, you can use `docs/1` to inject documentation:\n\n    @doc \"Supported options:\\n#{NimbleOptions.docs(@options_schema)}\"","ref":"NimbleOptions.html#docs/2"},{"type":"function","title":"Options - NimbleOptions.docs/2","doc":"* `:nest_level` - an integer deciding the \"nest level\" of the generated\n    docs. This is useful when, for example, you use `docs/2` inside the `:doc`\n    option of another schema. For example, if you have the following nested schema:\n\n        nested_schema = [\n          allowed_messages: [type: :pos_integer, doc: \"Allowed messages.\"],\n          interval: [type: :pos_integer, doc: \"Interval.\"]\n        ]\n\n    then you can document it inside another schema with its nesting level increased:\n\n        schema = [\n          producer: [\n            type: {:or, [:string, keyword_list: nested_schema]},\n            doc:\n              \"Either a string or a keyword list with the following keys:\\n\\n\" <>\n                NimbleOptions.docs(nested_schema, nest_level: 1)\n          ],\n          other_key: [type: :string]\n        ]","ref":"NimbleOptions.html#docs/2-options"},{"type":"function","title":"NimbleOptions.new!/1","doc":"Validates the given `schema` and returns a wrapped schema to be used with `validate/2`.\n\nIf the given schema is not valid, raises a `NimbleOptions.ValidationError`.","ref":"NimbleOptions.html#new!/1"},{"type":"function","title":"NimbleOptions.option_typespec/1","doc":"Returns the quoted typespec for any option described by the given schema.\n\nThe returned quoted code represents the **type union** for all possible\nkeys in the schema, alongside their type. Nested keyword lists are\nspec'ed as `t:keyword/0`.","ref":"NimbleOptions.html#option_typespec/1"},{"type":"function","title":"Usage - NimbleOptions.option_typespec/1","doc":"Because of how typespecs are treated by the Elixir compiler, you have\nto use `unquote/1` on the return value of this function to use it\nin a typespec:\n\n    @type option() :: unquote(NimbleOptions.option_typespec(my_schema))\n\nThis function returns the type union for a single option: to give you\nflexibility to combine it and use it in your own typespecs. For example,\nif you only validate part of the options through NimbleOptions, you could\nwrite a spec like this:\n\n    @type my_option() ::\n            {:my_opt1, integer()}\n            | {:my_opt2, boolean()}\n            | unquote(NimbleOptions.option_typespec(my_schema))\n\nIf you want to spec a whole schema, you could write something like this:\n\n    @type options() :: [unquote(NimbleOptions.option_typespec(my_schema))]","ref":"NimbleOptions.html#option_typespec/1-usage"},{"type":"function","title":"Example - NimbleOptions.option_typespec/1","doc":"schema = [\n      int: [type: :integer],\n      number: [type: {:or, [:integer, :float]}]\n    ]\n\n    @type option() :: unquote(NimbleOptions.option_typespec(schema))\n\nThe code above would essentially compile to:\n\n    @type option() :: {:int, integer()} | {:number, integer() | float()}","ref":"NimbleOptions.html#option_typespec/1-example"},{"type":"function","title":"NimbleOptions.validate/2","doc":"Validates the given `options` with the given `schema`.\n\nSee the module documentation for what a `schema` is.\n\nIf the validation is successful, this function returns `{:ok, validated_options}`\nwhere `validated_options` is a keyword list. If the validation fails, this\nfunction returns `{:error, validation_error}` where `validation_error` is a\n`NimbleOptions.ValidationError` struct explaining what's wrong with the options.\nYou can use `raise/1` with that struct or `Exception.message/1` to turn it into a string.","ref":"NimbleOptions.html#validate/2"},{"type":"function","title":"NimbleOptions.validate!/2","doc":"Validates the given `options` with the given `schema` and raises if they're not valid.\n\nThis function behaves exactly like `validate/2`, but returns the options directly\nif they're valid or raises a `NimbleOptions.ValidationError` exception otherwise.","ref":"NimbleOptions.html#validate!/2"},{"type":"type","title":"NimbleOptions.schema/0","doc":"A schema.\n\nSee the module documentation for more information.","ref":"NimbleOptions.html#t:schema/0"},{"type":"type","title":"NimbleOptions.t/0","doc":"The `NimbleOptions` struct embedding a validated schema.\n\nSee the [*Validating Schemas* section](#module-validating-schemas) in\nthe module documentation.","ref":"NimbleOptions.html#t:t/0"},{"type":"exception","title":"NimbleOptions.ValidationError","doc":"An error that is returned (or raised) when options are invalid.\n\nSince this is an exception, you can either raise it directly with `raise/1`\nor turn it into a message string with `Exception.message/1`.\n\nSee [`%NimbleOptions.ValidationError{}`](`__struct__/0`) for documentation on the fields.","ref":"NimbleOptions.ValidationError.html"},{"type":"function","title":"NimbleOptions.ValidationError.__struct__/0","doc":"The error struct.\n\nOnly the following documented fields are considered public. All other fields are\nconsidered private and should not be referenced:\n\n  * `:key` (`t:atom/0`) - The key that did not successfully validate.\n\n  * `:keys_path` (list of `t:atom/0`) - If the key is nested, this is the path to the key.\n\n  * `:value` (`t:term/0`) - The value that failed to validate. This field is `nil` if there\n    was no value provided.","ref":"NimbleOptions.ValidationError.html#__struct__/0"},{"type":"type","title":"NimbleOptions.ValidationError.t/0","doc":"","ref":"NimbleOptions.ValidationError.html#t:t/0"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v1.1.1 (2024-05-25) - Changelog","doc":"* Fix typespecs to avoid Dialyzer warnings\n  * Do not list default values as part of received options\n  * Mark structs in backticks for doc references\n  * List deprecations in docs","ref":"changelog.html#v1-1-1-2024-05-25"},{"type":"extras","title":"v1.1.0 (2023-12-13) - Changelog","doc":"* Add a `:type_spec` option to specify a quoted type spec for options.","ref":"changelog.html#v1-1-0-2023-12-13"},{"type":"extras","title":"v1.0.2 (2023-04-21) - Changelog","doc":"* Improve typespec generation for ranges and structs.","ref":"changelog.html#v1-0-2-2023-04-21"},{"type":"extras","title":"v1.0.1 (2023-03-13) - Changelog","doc":"* Make the `NimbleOptions.t/0` type *public* (instead of opaque). This helps with Dialyzer issues when ysing `NimbleOptions.new!/1` at compile time.","ref":"changelog.html#v1-0-1-2023-03-13"},{"type":"extras","title":"v1.0.0 (2023-03-08) - Changelog","doc":"* Add support for a `{:struct, struct_name}` type specifier\n  * Add support for the `:type_doc` option\n  * Turn `t:NimbleOptions.t/0` into an *opaque* type","ref":"changelog.html#v1-0-0-2023-03-08"},{"type":"extras","title":"v0.5.1 (2020-11-05) - Changelog","doc":"* Support generating typespecs for `:tuple`, `:map`, and `{:map, key, value}` options","ref":"changelog.html#v0-5-1-2020-11-05"},{"type":"extras","title":"v0.5.0 (2020-09-20) - Changelog","doc":"* Support `:map` which accepts the same `:keys` specification as keyword lists\n  * Normalize all error messages to include the key and expected value out of the box\n  * Do not nest options when rendered in Markdown and make sure multiline content is properly indented\n  * Handle \\r\\n style of line breaks in docs\n  * Automatically add types to generated docs\n  * Support lists of keyword lists in `:list`\n  * Add the `:reference` option type\n  * Add the `:tuple` option type","ref":"changelog.html#v0-5-0-2020-09-20"},{"type":"extras","title":"v0.4.0 (2020-10-28) - Changelog","doc":"* Add support for all enumerables in `{:in, choices}` instead of just lists. You can now do things such as `{:in, 1..10}`.\n  * Deprecate the `:rename_to` schema option and emit a warning when used.\n  * Remove the `{:one_of, choices}` type which was deprecated in v0.3.3.","ref":"changelog.html#v0-4-0-2020-10-28"},{"type":"extras","title":"v0.3.7 (2021-09-11) - Changelog","doc":"* Add `NimbleOptions.new!/1` to validate the schema once.","ref":"changelog.html#v0-3-7-2021-09-11"},{"type":"extras","title":"v0.3.6 (2021-07-13) - Changelog","doc":"* Add `:float` type.\n  * Fix docs generation when custom key type has no keys.","ref":"changelog.html#v0-3-6-2021-07-13"},{"type":"extras","title":"v0.3.5 (2020-11-09) - Changelog","doc":"* Add support for the `{:list, subtype}` type.","ref":"changelog.html#v0-3-5-2020-11-09"},{"type":"extras","title":"v0.3.4 (2020-11-05) - Changelog","doc":"* Support nested schemas in the `{:or, subtypes}` type as `{:or, [:string, keyword_list: [enabled: [type: :boolean]]]}`.\n  * Improve validation of the return value of `{:custom, module, function, args}` functions.\n  * Support options in `NimbleOptions.docs/2`. For now only the `:nest_level` option is supported.","ref":"changelog.html#v0-3-4-2020-11-05"},{"type":"extras","title":"v0.3.3 (2020-11-02) - Changelog","doc":"* Add the `{:or, subtypes}` type.\n  * Deprecate the `{:one_of, choices}` and replace it with `{:in, choices}`. Using `{:one_of, choices}` emits a warning now.","ref":"changelog.html#v0-3-3-2020-11-02"},{"type":"extras","title":"v0.3.2 (2020-10-26) - Changelog","doc":"* Fix a small bug with docs for nested schemas.","ref":"changelog.html#v0-3-2-2020-10-26"},{"type":"extras","title":"v0.3.1 (2020-10-25) - Changelog","doc":"* Return `:key` and `:value` on `%NimbleOptions.ValidationError{}` to allow programmatic use of errors.\n  * Validate default values according to the specified type.","ref":"changelog.html#v0-3-1-2020-10-25"},{"type":"extras","title":"v0.3.0 (2020-06-14) - Changelog","doc":"* **Breaking change**: return `{:error, %NimbleOptions.ValidationError{}}` tuples when there's a validation error in `NimbleOptions.validate/2` instead of `{:error, message}` (with `message` being a string). You can use `Exception.message/1` to turn the `NimbleOptions.ValidationError` struct into a string.\n  * Add the `:pid` type.","ref":"changelog.html#v0-3-0-2020-06-14"},{"type":"extras","title":"v0.2.1 (2020-05-08) - Changelog","doc":"* Add `NimbleOptions.validate!/2`.","ref":"changelog.html#v0-2-1-2020-05-08"},{"type":"extras","title":"v0.2.0 (2020-04-11) - Changelog","doc":"* Change the behavior of `NimbleOptions.docs/1` to accept a normal schema and produce documentation for that.\n  * Add support for `doc: false` as a schema option to hide an option or an option and its subsection.","ref":"changelog.html#v0-2-0-2020-04-11"},{"type":"extras","title":"v0.1.0 (2020-04-07) - Changelog","doc":"* First release.","ref":"changelog.html#v0-1-0-2020-04-07"},{"type":"extras","title":"License","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,51,46,48]}}